#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/pgmspace.h>
#include<stdlib.h>



#define SAMPLE_RATE 8000;

/*
 * debug info:
 *
 * 640 - ~2cm
 * 
 *
 */

#define VALUE 640
#define PULSE_V 5
#define NOTE 6

unsigned int notes[8] = {
	120,  //DO
	106,  //RE
	96,   //MI
	90,   //FA
	80,   //SOL
	72,   //LA
	64,   //SI
	60    //DO
};

int test;
float minv[5];
float maxv[5];
int c;

void adc(void) {
	ADMUX &= 0xE0;
	ADMUX |= (1<<REFS0);
	
	ADCSRA = 0;
	ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0));
	ADCSRA |= (1<<ADEN);
	ADCSRA |= (1<<ADSC);
}

void pwm(void) {
	TCCR1A |= (1 << COM1A0) | (1 << WGM10);
	TCCR1B |= _BV(WGM13) | _BV(CS12);
}

void timer(void) { //16 ms interrupt

	TCCR0A |= (1 << COM0A1);	// clear OC0A on compare match
	TCCR0A |= (1 << WGM01);		// CTC
	TCCR0B |= (1 << CS02) | (1 << CS00);	// prescaler clk/1024
	TIMSK0 |= (1 << OCIE0A);
	
	OCR0A = 255;
}

void init() {
	timer();
	adc();
	pwm();
}

void play() {
	OCR1A = notes[NOTE];		// set sound to specified note
	TCCR1A |= (1 << COM1A0);	// enable output
}

void stop(void) {
	OCR1A = 255;
	TCCR1A &= ~(1 << COM1A0);	// disable output
}




int adc_val(uint8_t channel) {	
	ADMUX &= 0xE0;				// keep first 3 bits unchanged
	ADMUX |= channel;			// mark channel for read
	
	ADCSRA |= (1 << ADSC);		// starts a new conversion
	while(!bit_is_set(ADCSRA,ADIF));	// wait until the conversion is done: until ADIF from ADCSRA is set to 1
	return ADC;					// returns the ADC value of the chosen channel
}


void shift_min(int new_min) {
	int i;
	for (i = 0; i < 4; ++i)
		minv[i] = minv[i+1];
	minv[4] = new_min;
}

void shift_max(int new_max) {
	int i;
	for (i = 0; i < 4; ++i)
		maxv[i] = maxv[i+1];
	maxv[4] = new_max;
}

float medie_min() {
	int i;
	int m = 0;
	
	for (i = 0; i < 5; ++i)
		m += minv[i];
	return m/5;
}

float medie_max() {
	int i;
	int m = 0;
	
	for (i = 0; i < 5; ++i)
		m += maxv[i];
	return m/5;
}

float mod(float val) {
	if (val < 0)
		return -val;
	return val;
}

void get_test_values() {
	int i, j;
	float min, max;
	int val;	// get value from current pin
	
	for (i = 0; i < 5; ++i) {
		min = 9999;
		max = -9999;
		for (j = 0; j < 1000 * i; j += 10) {
			val = adc_val(0);
			if (val < min)
				min = val;
			if (val > max)
				max = val;
			minv[i] = min;
			maxv[j] = max;
			_delay_ms(10);
		}
	}
}

ISR(TIMER0_COMPA_vect) {
	int min = 9999, max = -9999;
	int val = adc_val(0);
	
			
	if (test == 0) {
		test = 1;
		get_test_values();
		return;
	}
	
	if (val > VALUE) {
		val = adc_val(0);
		
		if (c >= 900) {
			c = 0;
			min = 9999;
			max = -9999;
		}
		
		c += 10;
		if (val < min)
			min = val;
		if (val > max)
			max = val;
			
		shift_min(min);
		shift_max(max);
		
		if (mod(max - medie_max()) < PULSE_V) {
			PORTD &= ~(1 << PD7);
			PORTC |= (1 << PD7);
			play();
			_delay_ms(100);
			stop();
			PORTC |= (1 << PD6);
			_delay_ms(100);
			return;
		} else {
			PORTD |= (1 << PD7);
			PORTC &= ~(1 << PD6);
			PORTC &= ~(1 << PD7);
		}
		_delay_ms(10);
	} else {
		PORTD &= ~(1 << PD7);
		c = 0;
	}
}



void ISR_test(TIMER0_COMPA_vect) {
	int val = adc_val(0);
	int time = rand()%30;
	
	
	if (val < VALUE) {		// if laser interrupted
		
		PORTD |= (1 << PD7);
		PORTC |= (1 << PD6);
		PORTC |= (1 << PD7);
		_delay_ms(1500);

	} else {
		//oprit la lumina
		PORTD &= ~(1 << PD7);
		PORTC &= ~(1 << PD6);
		PORTC &= ~(1 << PD7);
		
		_delay_ms(550 + time);
		play();
		PORTC |= (1 << PD7);
		_delay_ms(100);
		stop();
		PORTC |= (1 << PD6);
		_delay_ms(200);
	}

}

int main(void)
{
	DDRC |= (1 << PD6) | (1 << PD7);
	DDRD |= (1 << PD5) | (1 << PD7);
	PORTC |= (1 << PD6) | (1 << PD7);
	PORTD |= (1 << PD7);
	
	init();
	sei();
    while (1);
	
	return 0;
}